require 'json'
require 'uu/os/lang/hash_converter'

module UU
  module OS
    module Util

      # Module providing method for loading configuration.
      module ConfigurationLoader

        # Loads configuration (in properties or JSON format).
        # @param [String, IO] source Configuration source. Accepted are:
        #    * Absolute or relative path to configuration file.
        #    * IO providing configuration content.
        #    * String with configuration content.
        # @return [Hash<String, Object>] Loaded configuration.
        def load_configuration(source)
          ConfigurationLoader.load_configuration(source)
        end

        # @see #load_configuration
        # @private
        def self.load_configuration(source) # :nodoc:
          hash_cls = Class.new(Hash)
          hash_cls.send(:include, UU::OS::Lang::HashConverter)
          result = hash_cls.new
          return result if source.nil?

          # Get actual configuration content
          if source.kind_of?(String) && File.exists?(source)
            source = File.read(source)
          elsif source.respond_to?(:read)
            source = source.read
          end
          # We have to duplicate source, because strip
          # is not available on frozen string.
          source = source.dup.strip
          return result if source.empty?

          # Parse from JSON or property file
          if source =~ /^{.*}$/
            result.merge!(JSON.parse(source))
          else
            # Unify line breaks
            source.gsub!(/\r\n?/, "\n")
            idx = 0
            source.each_line do |line|
              idx += 1
              # Remove whitespaces
              line = line.strip
              # Skip comment lines
              next if line =~ /^#/
              # Split key and values (not escaped =)
              parts = line.split(/([^\\]=)/)
              raise "invalid property format on line #{idx}" if parts.size != 3
              key = (parts[0] << parts[1][0]).strip
              value = parts[2].strip
              result[key] = value
            end
          end
          return result
        end

      end

    end
  end
end
